'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.astToObj = astToObj;
exports.createInitialStateFromTypeInfo = createInitialStateFromTypeInfo;
exports.getTypeAst = getTypeAst;
exports.flowFakerSync = flowFakerSync;

var _babylon = require('babylon');

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _flowExeca = require('flow-execa');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// import blog from 'babel-log'

function _flowFaker(ast) {
  if (!ast) {
    return {};
  }

  const output = astToObj(getTypeAst(ast));
  const result = createInitialStateFromTypeInfo(output);
  return result;
}

function typeToValue(typeInfo) {
  switch (typeInfo) {
    case 'number':
      return 0;
    case 'string':
      return '';
    case 'boolean':
      return false;
    default:
      return null;
  }
}

function getType(node) {
  const nodeType = node.type;

  if (!nodeType) {
    return null;
  }

  if (nodeType === 'ObjectTypeAnnotation') {
    const props = node.properties;
    return props.reduce((acc, prop) => {
      const key = prop.key.name;
      return prop.optional ? acc : _extends({}, acc, { [key]: getType(prop.value) });
    }, {});
  }

  if (nodeType === 'ArrayTypeAnnotation') {
    return [];
  }

  if (nodeType === 'GenericTypeAnnotation') {
    if (node.typeParameters && node.typeParameters.params.length > 0) {
      const params = node.typeParameters.params;
      return params.reduce((arr, param) => {
        return [...arr, getType(param)];
      }, []);
    }
  }

  if (nodeType === 'NullableTypeAnnotation') {
    return null;
  }

  if (nodeType === 'StringTypeAnnotation') {
    return 'string';
  }

  if (nodeType === 'NumberTypeAnnotation') {
    return 'number';
  }

  if (nodeType === 'BooleanTypeAnnotation') {
    return 'boolean';
  }

  return nodeType;
}

function isNodeLike(obj) {
  return typeof obj.type === 'string' || typeof obj.kind === 'string';
}

function isPathLike(obj) {
  return isNodeLike(obj) && {}.hasOwnProperty.call(obj, 'node');
}

// eslint-disable-next-line import/exports-last
function astToObj(input) {
  if (isPathLike(input)) {
    input = input.node;
  }

  return getType(input);
}

// eslint-disable-next-line flowtype/no-weak-types
function createInitialStateFromTypeInfo(info) {
  if (!info || typeof info === 'string') {
    return typeToValue(info);
  }

  if (Array.isArray(info)) {
    return [];
  }

  return Object.keys(info).reduce((acc, key) => {
    // $FlowFixMe
    return _extends({}, acc, { [key]: createInitialStateFromTypeInfo(info[key]) });
  }, {});
}

function getTypeAst(input) {
  const typeString = `type X = ${input}`;

  const ast = (0, _babylon.parse)(typeString, {
    plugins: ['flow'],
    sourceType: 'module'
  });

  let typeAST = {};

  (0, _babelTraverse2.default)(ast, {
    TypeAlias(nodePath) {
      // if (nodePath.get('id').isIdentifier({ name: 'X' })) { }
      // nodeのidは`X`が必ず保証されてる
      typeAST = nodePath.get('right');
    }
  });

  return typeAST;
}

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (filePath, { row, column, cwd = process.cwd() }) {
    const ast = yield (0, _flowExeca.getTypeFromFile)(cwd, filePath, row, column);
    return _flowFaker(ast);
  });

  function flowFaker(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return flowFaker;
})();

function flowFakerSync(filePath, { row, column, cwd = process.cwd() }) {
  const ast = (0, _flowExeca.getTypeFromFileSync)(cwd, filePath, row, column);
  return _flowFaker(ast);
}